/* db.c */

/*
    NUT nutrition software 
    Copyright (C) 2004 Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "db.h"
#include "meal.h"
#include "options.h"
#include "util.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <ctype.h>
 
#ifndef NUTDIR
char nutdir1[]=".nutdb";
#else
char nutdir1[]=NUTDIR ;
#endif

char USDAjoined1[] = "sr16-1.nut";
char globalFOOD1[] = "FOOD_DES.txt";
char globalNUT1[] = "NUT_DATA.txt";
char globalWEIGHT1[] = "WEIGHT.txt";
char localFOOD1[] = "FOOD_DES.txt";
char localNUT1[] = "NUT_DATA.txt";
char localWEIGHT1[] = "WEIGHT.txt";
char foodfile1[] = "food.db";
char mealfile1[] = "meal.db";
char theusualfile1[] = "theusual.db";
char optionsfile1[] = "options.db";
#ifndef DOS
char versionfile1[] = ".version";
#else
char versionfile1[] = "version";
#endif

char USDAjoined[2000];
char globalFOOD[2000];
char globalNUT[2000];
char globalWEIGHT[2000];
char localFOOD[2000];
char localNUT[2000];
char localWEIGHT[2000];
char foodfile[2000];
char mealfile[2000];
char theusualfile[2000];
char optionsfile[2000];
char versionfile[2000];
char nutdir[2000];

int mealdb_mealsperday;

FILE *fp;

void read_food_files()
{
 read_joined_db(USDAjoined);
 if ( version(0) < 9 ) remove(optionsfile);
 read_options_db(1);
 read_FOOD(globalFOOD);
 read_NUT(globalNUT);
 read_WEIGHT(globalWEIGHT);
 read_FOOD(localFOOD);
 read_NUT(localNUT);
 read_WEIGHT(localWEIGHT);
 make_food_index();
 read_meal_db();
 read_theusual_db();
 full_meal_reindexing();
 write_food_db();
 write_meal_db();
 write_theusual_db();
 write_options_db();
}

void read_joined_db(char *textfile) 
{
char buffer[700];
char *bufptr;
char *fieldptr;
int count, junk, foodcount = 0;
float grams2;
if ((fp = fopen(textfile,"rb")) == NULL)
 {
 printf("Can't open food file \"%s\" to create food database.\n",textfile);
 printf("Press <enter> to continue...\n");
 junk = get_int();
 abort();
 }
printf("\nReading food file \"%s\"\n",textfile);
printf("to create food database \"%s\"...\n",foodfile);
while (fgets(buffer,700,fp) != NULL)
 {
 if ((new_food = malloc(sizeof(struct food))) == NULL)
  {
  printf("We are out of memory.  Bummer.\n");
  abort();
  }
 bufptr = buffer ;
 fieldptr = bufptr;
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ; bufptr++;
 new_food->ndb_no = atoi(fieldptr);
 fieldptr = new_food->name;
 while (*bufptr != '^')
  {
  *fieldptr++ = *bufptr++ ;
  }
 *fieldptr = '\0' ; bufptr ++;
 while (*bufptr != '^' ) bufptr++ ; /* Bypass scientific name */
 bufptr++ ;
 fieldptr = bufptr ;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr++ ;
 new_food->refuse = (float) atoi(fieldptr);
 fieldptr = bufptr ;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr++ ;
 new_food->prot_cal_factor = (float) atof(fieldptr);
 fieldptr = bufptr ;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr++ ;
 new_food->fat_cal_factor = (float) atof(fieldptr);
 fieldptr = bufptr ;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr++ ;
 new_food->cho_cal_factor = (float) atof(fieldptr);
 for (count = 0; count < NUTRIENT_COUNT - DERIVED; count++)
  {
  fieldptr = bufptr ;
  while (*bufptr != '^' ) bufptr++ ;
  *bufptr = '\0'; bufptr++ ;
  new_food->nutrient[count] = (float) atof(fieldptr);
  }
 fieldptr = bufptr;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr++ ;
 new_food->grams = (float) atof(fieldptr);
 if (new_food->grams == 0) new_food->grams = 100;
 fieldptr = bufptr;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr ++ ;
 strncpy(new_food->serving,fieldptr,50); new_food->serving[50] = '\0';
 fieldptr = bufptr;
 while (*bufptr != '^' ) bufptr++ ;
 *bufptr = '\0'; bufptr++ ;
/* is second household weight smaller? */
 grams2 = (float) atof(fieldptr);
 if (grams2 < new_food->grams && grams2 > 0)
  {
  new_food->grams = grams2;
  fieldptr = bufptr;
  while (*bufptr != '\n' ) bufptr++ ;
  *bufptr = '\0';
  strncpy(new_food->serving,fieldptr,50); new_food->serving[50] = '\0';
  }
 else
  {
  fieldptr = bufptr;
  while (*bufptr != '\n' ) bufptr++ ;
  *bufptr = '\0';
  }
 compute_derived_fields(new_food);
 junk = order_new_food();
 if (++foodcount % 500 == 0) printf("Read %d foods...\n", foodcount);
 }
fclose(fp);
}

void read_FOOD(char *textfile)
{
struct food *food_ptr;
char buffer[400];
char *bufptr;
char *fieldptr;
int count, junk, ndbno;
if ((fp = fopen(textfile,"rb")) == NULL) return;
printf("\nReading food description file \"%s\"\n",textfile);
printf("to update food database \"%s\"...\n",foodfile);
while (fgets(buffer,400,fp) != NULL)
 {
 bufptr = buffer;
 bufptr++;
 fieldptr = bufptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++; *bufptr = '\0' ;
 ndbno = atoi(fieldptr);
 food_ptr = find_ndbno_ptr(ndbno);
 new_food = NULL;
 if (food_ptr == NULL)
  {
  if ((new_food = malloc(sizeof(struct food))) == NULL)
   {
   printf("We are out of memory.  Bummer.\n");
   abort();
   }
  new_food->ndb_no = ndbno;
  for (count = 0; count < NUTRIENT_COUNT; count++) new_food->nutrient[count] = 0;
  if (ndbno >= options.next_recipe) options.next_recipe = ndbno + 1;
  food_ptr = new_food;
  }

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ; 
 
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 fieldptr = bufptr;
 fieldptr++; fieldptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++;
 strcpy(food_ptr->name,fieldptr);

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 fieldptr = bufptr;
 fieldptr++;
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;
 food_ptr->refuse = atof(fieldptr);

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 fieldptr = bufptr;
 fieldptr++;
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;
 food_ptr->prot_cal_factor = atof(fieldptr);

 fieldptr = bufptr;
 fieldptr++;
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;
 food_ptr->fat_cal_factor = atof(fieldptr); 

 fieldptr = bufptr;
 fieldptr++;
 food_ptr->cho_cal_factor = atof(fieldptr);
 if (food_ptr == new_food) junk = order_new_food();
 compute_derived_fields(food_ptr);
 }
fclose(fp);
}

void read_NUT(char *textfile)
{
struct food *food_ptr;
char buffer[400];
char *bufptr;
char *fieldptr;
int ndbno, nutrno;
float value;
if ((fp = fopen(textfile,"rb")) == NULL) return;
printf("\nReading nutrient file \"%s\"\n",textfile);
printf("to update food database \"%s\"...\n",foodfile);
while (fgets(buffer,400,fp) != NULL)
 {
 bufptr = buffer;
 bufptr++;
 fieldptr = bufptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++; *bufptr = '\0' ;
 ndbno = atoi(fieldptr);
 food_ptr = find_ndbno_ptr(ndbno);
 if (food_ptr == NULL) continue;

 fieldptr = bufptr;
 fieldptr++; fieldptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0';
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++; *bufptr = '\0' ;
 nutrno = atof(fieldptr);

 fieldptr = bufptr;
 fieldptr++;                        
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;
 value = atof(fieldptr);

 if (LookupNut[nutrno] != -1)
  {
  food_ptr->nutrient[LookupNut[nutrno]] = value;
  compute_derived_fields(food_ptr);
  }
 }
fclose(fp);
}

void read_WEIGHT(char *textfile)
{
struct food *food_ptr;
char buffer[400];
char textbuf[100];
char *bufptr;
char *fieldptr;
int ndbno;
float value;
if ((fp = fopen(textfile,"rb")) == NULL) return;
printf("\nReading food weight file \"%s\"\n",textfile);
printf("to update food database \"%s\"...\n",foodfile);
while (fgets(buffer,400,fp) != NULL)
 {
 bufptr = buffer;
 bufptr++;
 fieldptr = bufptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++; *bufptr = '\0' ;
 ndbno = atoi(fieldptr);
 food_ptr = find_ndbno_ptr(ndbno);
 if (food_ptr == NULL) continue;

 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;

 fieldptr = bufptr;
 fieldptr++;                        
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;
 value = atof(fieldptr);

 fieldptr = bufptr;
 fieldptr++; fieldptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++;
 while (*bufptr != '~') bufptr++;
 *bufptr = '\0' ; bufptr++; *bufptr = '\0';
 
 sprintf(textbuf,"%g %-s",value,fieldptr);
 strcpy(food_ptr->serving,textbuf);

 fieldptr = bufptr;
 fieldptr++;                        
 while (*bufptr != '^') bufptr++;
 *bufptr = '\0' ;
 food_ptr->grams = atof(fieldptr);
 }
fclose(fp);
}

int read_food_db()
{
int block, foodcount = 0;
struct food *food_ptr = &food_root;
if ((fp = fopen(foodfile,"rb")) == NULL) return 0;
for ( ; ; )
 {
 if ((new_food = malloc(sizeof(struct food))) == NULL)
  {
  printf("Bad news.  We are out of memory...");
  abort();
  }
 if ((block = fread(new_food,sizeof(struct food),1,fp)) != 0)
  {
  food_ptr->next = new_food;
  food_ptr = new_food;
  foodcount++;
  }
 else break;
 }
fclose(fp);
make_food_index();
return 1;
}

void read_meal_db()
{
int block;
struct meal *meal_ptr = &meal_root;
if ((fp = fopen(mealfile,"rb")) == NULL)
 {
 write_meal_db();
 return;
 }
mealdb_mealsperday = 0;
for ( ; ; )
 {
 if ((new_meal = malloc(sizeof(struct meal))) == NULL)
  {
  printf("Bad news.  We are out of memory...");
  abort();
  }
 if ((block = fread(new_meal,sizeof(struct meal),1,fp)) != 0)
  {
  meal_ptr->next = new_meal;
  meal_ptr = new_meal;
  if (meal_ptr->meal > mealdb_mealsperday) mealdb_mealsperday = meal_ptr->meal;
  }
 else break;
 }
fclose(fp);
}

void read_theusual_db()
{
int block;
struct meal *theusual_ptr = &theusual_root;
if ((fp = fopen(theusualfile,"rb")) == NULL)
 {
 write_theusual_db();
 return;
 }
for ( ; ; )
 {
 if ((new_theusual = malloc(sizeof(struct meal))) == NULL)
  {
  printf("Bad news.  We are out of memory...");
  abort();
  }
 if ((block = fread(new_theusual,sizeof(struct meal),1,fp)) != 0)
  {
  theusual_ptr->next = new_theusual;
  theusual_ptr = new_theusual;
  }
 else break;
 }
fclose(fp);
}

void write_meal_db()
{
int block;
struct meal *meal_ptr = &meal_root;
if ((fp = fopen(mealfile,"wb")) == NULL)
 {
 printf("Can't open meal database %s to write.\n",mealfile);
 abort();
 }
while (meal_ptr -> next != NULL)
 {
 meal_ptr = meal_ptr -> next;
 if ((block = fwrite(meal_ptr,sizeof(struct meal),1,fp)) != 1)
  {
  printf("Cannot write to the disk.  Aborting...\n");
  abort();
  }
 }
fclose(fp);
}

void write_theusual_db()
{
int block;
struct meal *theusual_ptr = &theusual_root;
if ((fp = fopen(theusualfile,"wb")) == NULL)
 {
 printf("Can't open theusual database %s to write.\n",theusualfile);
 abort();
 }
while (theusual_ptr -> next != NULL)
 {
 theusual_ptr = theusual_ptr -> next;
 if ((block = fwrite(theusual_ptr,sizeof(struct meal),1,fp)) != 1)
  {
  printf("Cannot write to the disk.  Aborting...\n");
  abort();
  }
 }
fclose(fp);
}

void write_food_db()
{
int block;
struct food *food_ptr = &food_root;
if ((fp = fopen(foodfile,"wb")) == NULL)
 {
 printf("Can't open food database %s to write.\n",foodfile);
 abort();
 }
while (food_ptr -> next != NULL)
 {
 food_ptr = food_ptr -> next;
 if ((block = fwrite(food_ptr,sizeof(struct food),1,fp)) != 1)
  {
  printf("Cannot write to the disk.  Aborting...\n");
  abort();
  }
 }
fclose(fp);
}

void write_recipe(struct food *food_ptr)
{
char buffer[300], numbuf[11], textbuf[100];
int block, c, i;
float servnum;
if ((fp = fopen(localFOOD,"ab")) == NULL)
 {
 printf("Can't open recipe description file %s to append.\n",localFOOD);
 abort();
 }
strcpy(buffer,"~\0");
sprintf(numbuf,"%d",food_ptr->ndb_no);
strcat(buffer,numbuf);
strcat(buffer,"~^~9999~^~\0");
strcpy(textbuf,food_ptr->name);
c = strlen(textbuf);
for (i = 1; i < c; i++) textbuf[i] = tolower(textbuf[i]);
strcat(buffer,textbuf);
strcat(buffer,"~^~\0");
strcat(buffer,food_ptr->name);
strcat(buffer,"~^~~^~~^~~^~~^\0");
sprintf(numbuf,"%d",food_ptr->refuse);
strcat(buffer,numbuf);
strcat(buffer,"^~~^^\0");
if (food_ptr->nutrient[PROCNT] > 0) sprintf(numbuf,"%1.2f",food_ptr->nutrient[PROT_KCAL] / food_ptr->nutrient[PROCNT]);
else sprintf(numbuf,"%d",0);
strcat(buffer,numbuf);
strcat(buffer,"^\0");
if (food_ptr->nutrient[FAT] > 0) sprintf(numbuf,"%1.2f",food_ptr->nutrient[FAT_KCAL] / food_ptr->nutrient[FAT]);
else sprintf(numbuf,"%d",0);
strcat(buffer,numbuf);
strcat(buffer,"^\0");
if (food_ptr->nutrient[CHOCDF] > 0) sprintf(numbuf,"%1.2f",food_ptr->nutrient[CHO_KCAL] / food_ptr->nutrient[CHOCDF]);
else sprintf(numbuf,"%d",0);
strcat(buffer,numbuf);
strcat(buffer,"\n");
c = strlen(buffer);
if ((block = fwrite(buffer,c,1,fp)) != 1)
 {
 printf("Cannot write to the disk.  Aborting...\n");
 abort();
 }
fclose(fp);

if ((fp = fopen(localNUT,"ab")) == NULL)
 {
 printf("Can't open recipe nutrient file %s to append.\n",localNUT);
 abort();
 }
for (i=0; i < NUTRIENT_COUNT - DERIVED; i++)
 {
 if (food_ptr->nutrient[i] == 0) continue;
 strcpy(buffer,"~\0");
 sprintf(numbuf,"%d",food_ptr->ndb_no);
 strcat(buffer,numbuf);
 strcat(buffer,"~^~\0");
 sprintf(numbuf,"%d",LookupNutrNo[i]);
 strcat(buffer,numbuf);
 strcat(buffer,"~^\0");
 sprintf(numbuf,"%1.3f",food_ptr->nutrient[i]);
 strcat(buffer,numbuf);
 strcat(buffer,"^0^^~4~^~~^~~^~~^^^^^^^~~^^~~\n");
 c = strlen(buffer);
 if ((block = fwrite(buffer,c,1,fp)) != 1)
  {
  printf("Cannot write to the disk.  Aborting...\n");
  abort();
  }
 }
fclose(fp);

if ((fp = fopen(localWEIGHT,"ab")) == NULL)
 {
 printf("Can't open recipe weight file %s to append.\n",localWEIGHT);
 abort();
 }
strcpy(buffer,"~\0");
sprintf(numbuf,"%d",food_ptr->ndb_no);
strcat(buffer,numbuf);
strcat(buffer,"~^1^\0");
servnum = atof(food_ptr->serving);
if (servnum == 0) servnum = 1;
sprintf(numbuf,"%g",servnum);
strcat(buffer,numbuf);
strcat(buffer,"^~\0");
strcpy(textbuf,food_ptr->serving);
c = strspn(textbuf," 0123456789.");
strcat(buffer,textbuf+c);
strcat(buffer,"~^\0");
sprintf(numbuf,"%g",food_ptr->grams);
strcat(buffer,numbuf);
strcat(buffer,"^^\n");
c = strlen(buffer);
if ((block = fwrite(buffer,c,1,fp)) != 1)
 {
 printf("Cannot write to the disk.  Aborting...\n");
 abort();
 }
fclose(fp);
}

void write_options_db()
{
int block;
if ((fp = fopen(optionsfile,"wb")) == NULL)
 {
 printf("Can't open options database \"%s\" to write.\n",optionsfile);
 printf("Press <enter> to continue...");
 block = get_int();
 abort();
 }
if ((block = fwrite(&options,sizeof(struct opt),1,fp)) != 1)
 {
 printf("Cannot write to the disk.  Aborting...\n");
 abort();
 }
fclose(fp);
}

void read_options_db(int dontwrite)
{
int block, need_to_find_next_recipe = 0;
struct food *food_ptr = &food_root;
initialize_options();
if ((fp = fopen(optionsfile,"rb")) == NULL) 
 {
 write_options_db();
 if (! dontwrite) need_to_find_next_recipe = 1;
 }
if ((fp = fopen(optionsfile,"rb")) == NULL) 
 {
 printf("Can't open options database \"%s\" to read.\n",optionsfile);
 printf("Press <enter> to continue...");
 block = get_int();
 abort();
 }
fread(&options,sizeof(struct opt),1,fp);
fclose(fp);
options.temp_meal_root = &meal_root;
auto_cal(&options.abnuts[ENERC_KCAL],0);
delete_meals(options.delopt);
if (! dontwrite) write_meal_db();
if (need_to_find_next_recipe) while (food_ptr->next != NULL)
 {
 food_ptr = food_ptr->next;
 if (food_ptr->ndb_no >= options.next_recipe) options.next_recipe = food_ptr->ndb_no + 1;
 }
}

void make_filenames(void)
{
#ifdef DOS
 sprintf(nutdir,"%s","NUTDB");
 mkdir (nutdir, 0700 );
 sprintf(USDAjoined,"%s/%s","NUTSR161",USDAjoined1);
 sprintf(globalFOOD,"%s/%s","NUTSR161",globalFOOD1);
 sprintf(globalNUT,"%s/%s","NUTSR161",globalNUT1);
 sprintf(globalWEIGHT,"%s/%s","NUTSR161",globalWEIGHT1);
 sprintf(localFOOD,"%s/%s","NUTDB",localFOOD1);
 sprintf(localNUT,"%s/%s","NUTDB",localNUT1);
 sprintf(localWEIGHT,"%s/%s","NUTDB",localWEIGHT1);
 sprintf(foodfile,"%s/%s","NUTDB",foodfile1);
 sprintf(mealfile,"%s/%s","NUTDB",mealfile1);
 sprintf(theusualfile,"%s/%s","NUTDB",theusualfile1);
 sprintf(optionsfile,"%s/%s","NUTDB",optionsfile1);
 sprintf(versionfile,"%s/%s","NUTDB",versionfile1);
#else
sprintf(nutdir,"%s/%s",getenv("HOME"),nutdir1);
mkdir (nutdir, 0700 );
sprintf(USDAjoined,"%s/%s",FOODDIR,USDAjoined1);
sprintf(globalFOOD,"%s/%s",FOODDIR,globalFOOD1);
sprintf(globalNUT,"%s/%s",FOODDIR,globalNUT1);
sprintf(globalWEIGHT,"%s/%s",FOODDIR,globalWEIGHT1);
sprintf(localFOOD,"%s/%s/%s",getenv("HOME"),nutdir1,localFOOD1);
sprintf(localNUT,"%s/%s/%s",getenv("HOME"),nutdir1,localNUT1);
sprintf(localWEIGHT,"%s/%s/%s",getenv("HOME"),nutdir1,localWEIGHT1);
sprintf(foodfile,"%s/%s/%s",getenv("HOME"),nutdir1,foodfile1);
sprintf(mealfile,"%s/%s/%s",getenv("HOME"),nutdir1,mealfile1);
sprintf(theusualfile,"%s/%s/%s",getenv("HOME"),nutdir1,theusualfile1);
sprintf(optionsfile,"%s/%s/%s",getenv("HOME"),nutdir1,optionsfile1);
sprintf(versionfile,"%s/%s/%s",getenv("HOME"),nutdir1,versionfile1);
#endif
}

int version(int level)
{
char release_string[9];
int majrelease;
int minrelease;
if ((fp = fopen(versionfile,"rb")) == NULL) return 0;
fread(&release_string,8,1,fp);
fclose(fp);
majrelease = atoi(strtok(release_string,"."));
minrelease = atoi(strtok(NULL,"."));
return level > 0 ? minrelease : majrelease;
}

void write_version()
{
char release_string[]=VERSIONQ;
int block;
if ((fp = fopen(versionfile,"wb")) == NULL)
 {
 printf("Can't write version number to disk in \"%s\".\n",versionfile);
 printf("Press <enter> to continue...");
 block = get_int();
 abort();
 }
if ((block = fwrite(release_string,strlen(release_string)+1,1,fp)) != 1)
 {
 printf("Cannot write to the disk.  Aborting...\n");
 abort();
 }
fclose(fp);
}

