/* food.c */

/*
    NUT nutrition software 
    Copyright (C) 2004 Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "food.h"
#include "meal.h"
#include "util.h"
#include "options.h"
#include "nutrient.h"
#include "abbrev.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* The following adjusts 100% of sat, mono, & poly to = 100% of fat */
#define FATTY_ACID_COMPONENT 0.9

struct food food_root, food_work, *new_food, recipe_root, *new_recipe, *FoodIndex[MAX_FOOD];

int DVMap[] =
{
DV_COUNT,ENERC_KCAL,
FAT,FASAT,FAMS,FAPU,OMEGA6,LA,AA,OMEGA3,ALA,EPA,DHA,CHOLE,CHOCDF,FIBTG,PROCNT,
VITA_IU,THIA,RIBF,NIA,PANTAC,VITB6A,FOL,VITB12,VITC,VITD,VITE,VITK,
CA,CU,FE,MG,MN,P,K,SE,NA,ZN
};

int DV1Map[] =
{
DV_COUNT,ENERC_KCAL,
FAT,FASAT,FAMS,FAPU,OMEGA6,LA,AA,OMEGA3,ALA,EPA,DHA,CHOLE,CHOCDF,FIBTG,PROCNT,
VITA_IU,THIA,RIBF,NIA,PANTAC,VITB6A,FOL,VITB12,VITC,VITD,VITE,VITK,
CA,CU,FE,MG,MN,P,K,SE,NA,ZN
};

int CarbAminoMap[] =
{
32,CHOCDF,FIBTG,STARCH,SUGAR,FRUS,GALS,GLUS,LACS,MALS,SUCS,ALC,
PROCNT,ADPROT,ALA_G,ARG_G,ASP_G,CYS_G,GLU_G,GLY_G,HISTN_G,HYP,ILE_G,
LEU_G,LYS_G,MET_G,PHE_G,PRO_G,SER_G,THR_G,TRP_G,TYR_G,VAL_G
};

int MiscMap[] =
{
24,ENERC_KJ,ASH,WATER,CAFFN,THEBRN,FOLAC,FOLFD,FOLDFE,RETOL,VITA_RAE,
TOCPHA,TOCPHB,TOCPHG,TOCPHD,CARTA,CARTB,CRYPX,LUT_ZEA,LYCPN,
CHOLE,PHYSTR,SITSTR,CAMD5,STID7
};

int SatMap[] =
{
15,FASAT,F4D0,F6D0,F8D0,F10D0,F12D0,F13D0,F14D0,F15D0,F16D0,
F17D0,F18D0,F20D0,F22D0,F24D0
};

int UnsatMap[] =
{
31,FAMS,F14D1,F15D1,F16D1,F16D1C,F17D1,F18D1,F18D1C,F20D1,F22D1,F24D1C,
FAPU,F18D2,F18D2CN6,F18D3,F18D3CN3,F18D3CN6,F18D4,F20D2CN6,F20D3,
F20D4,F20D5,F22D5,F22D6,
FATRN,FATRNM,F16D1T,F18D1T,FATRNP,F18D2I,F18D2CLA
};

int *ScreenMap[] =
{
DVMap,DV1Map,CarbAminoMap,MiscMap,SatMap,UnsatMap
};

float DVBase[NUTRIENT_COUNT];

float DV[NUTRIENT_COUNT];

int LookupNutrNo[NUTRIENT_COUNT - DERIVED];
int LookupNut[NUTRNO_ARRAYSIZE];

int FoodCount = 0;

void initializations()
{
int c;
for (c=0; c < NUTRIENT_COUNT; c++) DVBase[c] = 0;
DVBase[ENERC_KCAL] = 2000;
DVBase[PROCNT] = 50;
DVBase[FAT] = 65;
DVBase[CHOCDF] = 300;
DVBase[FIBTG] = 25;
DVBase[CA] = 1000;
DVBase[P] = 1000;
DVBase[FE] = 18;
DVBase[NA] = 2400;
DVBase[K] = 3500;
DVBase[MG] = 400;
DVBase[ZN] = 15;
DVBase[CU] = 2;
DVBase[MN] = 2;
DVBase[SE] = 70;
DVBase[VITA_IU] = 5000;
DVBase[VITE] = 30;
DVBase[VITK] = 80;
DVBase[THIA] = 1.5;
DVBase[RIBF] = 1.7;
DVBase[NIA] = 20;
DVBase[PANTAC] = 10;
DVBase[VITB6A] = 2;
DVBase[FOL] = 400;
DVBase[VITB12] = 6;
DVBase[VITC] = 60;
DVBase[FASAT] = 20;
DVBase[CHOLE] = 300;
DVBase[VITD] = 400;
DVBase[LA] = 10;
DVBase[AA] = .3;
DVBase[OMEGA6] = 10.3;
DVBase[ALA] = 1.3;
DVBase[EPA] = .3;
DVBase[DHA] = .3;
DVBase[OMEGA3] = 1.9;
DVBase[FAPU] = 12.2;
DVBase[FAMS] = 25.5;
for (c=0; c < NUTRNO_ARRAYSIZE; c++) LookupNut[c] = -1;
for (c=0; c < NUTRIENT_COUNT - DERIVED; c++) LookupNutrNo[c] = -1;
#include "lookup.h"
}

int order_new_food()
{
struct food *food_ptr = &food_root;
int foodnum = 0;
while (food_ptr->next != NULL && namestrcmp(new_food->name, food_ptr->next->name) >= 0)
 {
 food_ptr = food_ptr->next;
 foodnum++;
 }
new_food->next = food_ptr->next;
food_ptr->next = new_food;
return foodnum;
}

void order_new_food_already_sorted()
{
struct food *food_ptr = &food_root;
while (food_ptr->next != NULL) food_ptr = food_ptr->next;
new_food->next = NULL;
food_ptr->next = new_food;
}

int namestrcmp(char *name1, char *name2)
{
char namebuf1[61], namebuf2[61];
int count;
for (count = 0 ; count <= 60 ; count++)
 {
 if (name1[count] == ',') namebuf1[count] = '\a';
 else namebuf1[count] = name1[count]; 
 if (name2[count] == ',') namebuf2[count] = '\a';
 else namebuf2[count] = name2[count]; 
 }
return strcmp(namebuf1,namebuf2);
}

int substring_finder(char *food_name, char *substr)
{
int i;
char *token;
char substring[61];
strcpy(substring,substr);
token = strtok(substring, ", ");
while (token != NULL)
 {
 if (strstr(food_name,token) != NULL)
  {
  token = strtok(NULL,", ");
  continue;
  }
 for (i=0; i < Abbrev_Count; i++) if (strcmp(token,Abbreviations[i]) == 0) break;
 if (i == Abbrev_Count) return 0;
 if (strstr(food_name,Abbreviations[i % 2 == 0 ? i+1 : i-1]) == NULL) return 0;
 token = strtok(NULL,", ");
 }
return 1;
}

struct food *food_choice(char *screentitle, int usual)
{
struct food *food_ptr;
struct food *ptrlist[MAX_FOOD];
int count = 0, charcount, keysize, lines, junk;
char substring[61], key[61];
for ( ; ; )
 {
 key_decode(substring,key);
 if (strcmp(substring,"") == 0)
  {
  header(screentitle);
  spacer(0);
  if (strcmp(screentitle,"NUT:  Add Recipes") == 0) printf("\nType food name to add to recipe (or <enter> to quit):  ");
  else printf("\nType food name to select (or <enter> to quit):  ");
  get_string(substring,60);
  key[0] = '\0';
  key_clean();
  }
 if (substring[0] == '\0') return (struct food *) -1;
 for (charcount = 0 ; charcount < 60 ; charcount++) substring[charcount] = toupper(substring[charcount]);
 key_encode(substring,key);
 if (strncmp(substring,"THEUSUAL",8) == 0 && usual) return (struct food *) -2;
 header(screentitle);
 food_ptr = &food_root;
 keysize = strlen(key); lines = 0; count = 0;
 while ((food_ptr = food_ptr->next)) if (substring_finder(food_ptr->name,substring) && strncmp(key,food_ptr->name,keysize) == 0) ptrlist[++count] = food_ptr;
 while (count == 0)
  {
  key_clean();
  spacer(-1);
  printf("No food named \"%s\".  Try another (or <enter> to quit):  ",substring);
  get_string(substring,60);
  if (substring[0] == '\0') return (struct food *) 0;
  for (charcount = 0 ; charcount < 60 ; charcount++) substring[charcount] = toupper(substring[charcount]);
  key_encode(substring,key);
 if (strncmp(substring,"THEUSUAL",8) == 0 && usual) return (struct food *) -2;
  header(screentitle);
  food_ptr = &food_root;
  keysize = strlen(key); lines = 0; count = 0;
  while ((food_ptr = food_ptr->next)) if (substring_finder(food_ptr->name,substring) && strncmp(key,food_ptr->name,keysize) == 0) ptrlist[++count] = food_ptr;
  }
 if (count == 1) return ptrlist[1];
 if (count < 21)
  {
  for (lines = 1 ; lines <= count ; lines++) printf("%2d. %s\n",lines,ptrlist[lines]->name);
  spacer(lines - 2);
  printf("Type number of food category (\"b\" to go back, <enter> to quit):  ");
  get_string(key,60);
  junk = 0;
  if (strcmp(key,"b") == 0 || strcmp(key,"B") == 0) junk = -38;
  if (junk == -38) 
   {
   key_decode(substring,key);
   continue;
   }
  junk = atoi(key); 
  if (junk > 0 && junk <= count) 
   {
   key_encode(substring,ptrlist[junk]->name);
   return ptrlist[junk];
   }
  else return (struct food *) 0;   
  }
 if (count > 20) food_subcat(substring, key, ptrlist, count);
 }
}

void food_subcat(char *substring, char *key, struct food **ptrlist, int count)
{
char food_screen[2000];
char *fsptr[500], *thisfsptr, *bufptr, buffer[61], format[11], format100[11], tokenhead[61], junkstring[120];
char *col[10], colformat1[110], colformat2[110];
int i, l, s, subcount = 0, charcount, tokencount, nlength, tokenlength, tokenunique, headlength = 0, linecount, savelinecount = 0, tokenmax = 0, colcount;
for (linecount = 1 ; linecount <= 20 ; linecount++) fsptr[linecount] = food_screen + (61 * linecount);
for (linecount = 1 ; linecount <= count ; linecount++)
 {
 tokencount = 0;
 nlength = strlen(ptrlist[linecount]->name);
 for (charcount = 0 ; charcount <= nlength ; charcount++) if (ptrlist[linecount]->name[charcount] == ',') tokencount++;
 if (tokencount > tokenmax) tokenmax = tokencount;
 }
while (subcount < count && tokenmax > -1)
 {
 tokenmax--;
 subcount = 0;
 for (linecount = 1; subcount < count && linecount <= 20 ; linecount++)
  {
  subcount++;
  nlength = strlen(ptrlist[subcount]->name);
  thisfsptr = fsptr[linecount];
  tokencount = 0;
  for (charcount = 0 ; charcount <= nlength ; charcount++)
   {
   if (ptrlist[subcount]->name[charcount] == ',') tokencount++;
   if (tokencount == tokenmax) 
    {
    thisfsptr[charcount] = '\0';
    break;
    }
   thisfsptr[charcount] = ptrlist[subcount]->name[charcount];
   if (ptrlist[subcount]->name[charcount] == '\0') break;
   }
  if (linecount > 1 && 0 == strcmp(fsptr[linecount],fsptr[linecount-1])) linecount--;
  }
 savelinecount = --linecount;
 }
if (savelinecount > 1)
 {
 for (linecount = 1; linecount <= savelinecount ; linecount++) printf("%2d.  %s\n",linecount,fsptr[linecount]);
 spacer(savelinecount-1);
 printf("Type number of food category (\"b\" to go back, <enter> to quit):  ");
 get_string(junkstring,20);
 if (strcmp(junkstring,"b") == 0 || strcmp(junkstring,"B") == 0) 
  {
  key_take();
  return;
  }
 linecount = atoi(junkstring);
 if (linecount > 0 && linecount <= savelinecount) 
  {
  if (headlength < 1) strcpy(key,fsptr[linecount]);
  if (headlength > 0)
   {
   strcpy(key,tokenhead);
   strcat(key,fsptr[linecount]);
   }
  for (subcount = 1 ; subcount <= count ; subcount++) if (strcmp(key,ptrlist[subcount]->name) == 0) break;
  if (--subcount == count) strcat(key,",");
  }
 else
  {
  key[0] = '\0';
  substring[0] = '\0';
  }
 key_encode(substring,key);
 }
else
 {
 tokenhead[0] = '\0';
 for (tokenmax = 1 ; ; tokenmax++)
  {
  buffer[0] = '\0';
  tokenunique = 0;
  tokenlength = 0;
  for (subcount = 1 ; subcount <= count ; subcount++)
   {
   bufptr = ptrlist[subcount]->name;
   nlength = 0;
   for (tokencount = 1 ; tokencount <= tokenmax ; tokencount++)
    {
    bufptr += nlength;
    bufptr += strspn(bufptr,",");
    nlength = strcspn(bufptr,",");
    } 
   if (nlength > tokenlength) 
    {
    tokenlength = nlength;
    tokenunique++;
    strncpy(buffer,bufptr,nlength);
    buffer[nlength] = '\0';
    }
   else if (nlength < tokenlength) 
    {
    tokenunique++;
    strncpy(buffer,bufptr,nlength);
    buffer[nlength] = '\0';
    }
   else if (strncmp(buffer,bufptr,nlength) != 0)
    {
    tokenunique++;
    strncpy(buffer,bufptr,nlength);
    buffer[nlength] = '\0';
    }
   else;
   }
  if (tokenunique == 1)
   {
   strcat(buffer,",");
   strcat(tokenhead,buffer);
   }
  if (tokenunique != 1) break;
  }
 if (tokenhead[0] == '\0') savelinecount = 20;
 else savelinecount = 19; 
 headlength = strlen(tokenhead);
#ifndef DOS
 while ( savelinecount * (81 / (tokenlength + 5 ) ) < tokenunique )
#else
 while ( savelinecount * (80 / (tokenlength + 5 ) ) < tokenunique )
#endif
  {
  buffer[0] = '\0';
  tokenunique = 0 ;
  tokenlength--;
  for (subcount = 1 ; subcount <= count ; subcount++)
   {
   bufptr = ptrlist[subcount]->name + headlength;
   nlength = strcspn(bufptr,",");
   while (*(bufptr + nlength - 1) == ' ') nlength--;
   if (nlength > tokenlength) nlength = tokenlength;
   if (strncmp(buffer,bufptr,nlength) != 0)
    {
    tokenunique++;
    strncpy(buffer,bufptr,nlength);
    buffer[nlength] = '\0';
    }
   }
  }
 for (linecount = 0 ; linecount <= (tokenunique+2) ; linecount++) fsptr[linecount] = food_screen + ((tokenlength+1) * linecount);
 food_screen[0] = '\0';
 linecount = 0;
 for (subcount = 1 ; subcount <= count ; subcount++)
  { 
  bufptr = ptrlist[subcount]->name + headlength;
  nlength = (strcspn(bufptr,","));
  if (nlength > tokenlength) nlength = tokenlength;
  while (*(bufptr + nlength - 1) == ' ') nlength--;
  strncpy(fsptr[linecount + 1],bufptr,nlength);
  thisfsptr = fsptr[linecount + 1];
  thisfsptr[nlength] = '\0';
  if (strncmp(fsptr[linecount],fsptr[linecount + 1],nlength) != 0) linecount++;
  }
 sprintf(format,"%%2d. %%-%ds",tokenlength);
 sprintf(format100,"%%3d. %%-%ds",tokenlength);
 strcpy(colformat1,format);
 strcpy(colformat2,format);
 colcount = (linecount % savelinecount == 0 ? 0 : 1) + (linecount / savelinecount);
 if (colcount > 7)
  {
  spacer(-1);
  printf("Too many foods meet criterion.  Press <enter> to continue...");
  i = get_int();
  key_take();
  return;
  } 
 if (colcount < 8)
  {
  for (i = 2 ; i <= colcount ; i++)
   {
   if (i != colcount) strcat(colformat1," ");
   strcat(colformat2," ");
   if ((i * savelinecount) < 100 || tokenunique < 100)
    {
    if (i != colcount) strcat(colformat1,format);
    strcat(colformat2,format);
    }
   else 
    {
    if (i != colcount) strcat(colformat1,format100);
    strcat(colformat2,format100);
    }
   }
  strcat(colformat1,"\n");
  strcat(colformat2,"\n");
  col[colcount - 1] = colformat1;
  col[colcount] = colformat2;
  if (headlength > 0 ) printf("%-s\n",tokenhead);
  if ( linecount < savelinecount ) savelinecount = linecount;
  l = linecount ; s = savelinecount;
  for (i = 1 ; i <= savelinecount ; i++)
   {
   if      (i+(1*s)>l) printf(col[1],i,fsptr[i]);
   else if (i+(2*s)>l) printf(col[2],i,fsptr[i],i+s,fsptr[i+s]);
   else if (i+(3*s)>l) printf(col[3],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)]);
   else if (i+(4*s)>l) printf(col[4],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)]);
   else if (i+(5*s)>l) printf(col[5],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)]);
   else if (i+(6*s)>l) printf(col[6],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)]);
   else if (i+(7*s)>l) printf(col[7],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)],i+(6*s),fsptr[i+(6*s)]);
   else if (i+(8*s)>l) printf(col[8],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)],i+(6*s),fsptr[i+(6*s)],i+(7*s),fsptr[i+(7*s)]);
   else if (i+(9*s)>l) printf(col[9],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)],i+(6*s),fsptr[i+(6*s)],i+(7*s),fsptr[i+(7*s)],i+(8*s),fsptr[i+(8*s)]);
   else;
   }
  spacer(savelinecount < 19 ? savelinecount : 19);
  printf("Type number of food category (\"b\" to go back, <enter> to quit):  ");
  get_string(junkstring,20);
  if (strcmp(junkstring,"b") == 0 || strcmp(junkstring,"B") == 0) 
   {
   key_take();
   return;
   }
  i = atoi(junkstring);
  if (i > 0 && i <= linecount)
   {
   if (headlength > 0)
    {
    strcpy(key,tokenhead);
    strcat(key,fsptr[i]);
    }
   else strcpy(key,fsptr[i]);
   }
  else
   {
   key[0] = '\0';
   substring[0] = '\0';
   }
  key_encode(substring,key);
  }
 }
}

void food_show(struct food *food_ptr, float *ratio)
{
char servingstring[62];
int count;
if (*ratio > 1 || (*ratio > 0 && *ratio < 1))
 {
 sprintf(servingstring,"%6.2f x %-42s",*ratio,food_ptr->serving);
 servingstring[50] = '\0';
 }
printf("%-60s   Grams  %8.2f\n",food_ptr->name,food_ptr->grams * *ratio);
if (*ratio == 1) printf("Serving:  %-50s   Ounces  %7.2f\n",food_ptr->serving,*ratio * food_ptr->grams / 28.35);
else printf("Serving:  %-50s   Ounces  %7.2f\n",servingstring,*ratio * food_ptr->grams / 28.35);
printf("          %-50s   Water    %6.2f%%\n"," ",food_ptr->nutrient[WATER]);
if ( ! options.custom && options.screen == 0 ) printf("Percentages of \"Daily Values\" in this serving:                 Refuse   %3d%%\n\n",food_ptr->refuse);
if ( options.custom && options.screen == 0 )     printf("Percentages of customized DV in this serving:                  Refuse   %3d%%\n\n",food_ptr->refuse);
if ( options.screen > 0 )                 printf("Nutrients in this serving:                                     Refuse   %3d%%\n\n",food_ptr->refuse);
for (count = 0; count < NUTRIENT_COUNT; count++) food_work.nutrient[count] = *ratio * food_ptr->nutrient[count] * food_ptr->grams / 100; 
food_display(stdout);
spacer(20);
}

struct food *food_number(int i)
{
return FoodIndex[i];
}

int find_ndbno(int ndbno)
{
struct food *food_ptr = &food_root;
while (food_ptr->next != NULL)
 {
 food_ptr = food_ptr->next;
 if (food_ptr->ndb_no == ndbno) return food_ptr->food_no;
 }
return -1;
}

struct food *find_ndbno_ptr(int ndbno)
{
struct food *food_ptr = &food_root;
while (food_ptr->next != NULL)
 {
 food_ptr = food_ptr->next;
 if (food_ptr->ndb_no == ndbno) return food_ptr;
 }
return NULL;
}

void make_food_index()
{
struct food *food_ptr = &food_root;
FoodCount = 0;
while (food_ptr->next != NULL)
 {
 food_ptr = food_ptr->next;
 food_ptr->food_no = FoodCount;
 FoodIndex[FoodCount++] = food_ptr;
 }
}

int modify_food_index(int foodnum, struct food *food_ptr)
{
int count;
for (count = FoodCount; count > foodnum; count--) FoodIndex[count] = FoodIndex[count-1];
FoodIndex[foodnum] = food_ptr;
FoodCount++;
food_ptr = &food_root;
for (count = 0; count < FoodCount; count++)
 {
 food_ptr = food_ptr->next;
 food_ptr->food_no = count;
 }
return foodnum;
}

void clear_work()
{
int count;
for (count = 0; count < NUTRIENT_COUNT; count++) food_work.nutrient[count] = 0;
food_work.grams = 0;
food_work.refuse = 0;
}

void food_display(FILE *fp)
{
float pseudocals = 0, pctfat = 0, pctcarb = 0, pctprot = 0;
char carbprotfat[9];
if (food_work.nutrient[ENERC_KCAL] != 0)
 {
 pseudocals = food_work.nutrient[FAT_KCAL] + food_work.nutrient[CHO_KCAL] + food_work.nutrient[PROT_KCAL];
 pctfat = 100 * food_work.nutrient[FAT_KCAL] / pseudocals;
 pctcarb = 100 * food_work.nutrient[CHO_KCAL] / pseudocals;
 pctprot = 100 * food_work.nutrient[PROT_KCAL] / pseudocals;
 }
else
 {
 pctfat  = 0;
 pctcarb = 0;
 pctprot = 0;
 }

sprintf(carbprotfat,"%1.0f/%1.0f/%1.0f",pctcarb,pctprot,pctfat);
switch (options.screen)
 {
 case 1 : 
#ifndef DOS
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[ENERC_KCAL],food_work.nutrient[ENERC_KCAL],Unit[ENERC_KCAL],Nutrient[CHOCDF],food_work.nutrient[CHOCDF],Unit[CHOCDF],Nutrient[PROCNT],food_work.nutrient[PROCNT],Unit[PROCNT]);
  fprintf(fp,"Carb/Prot/Fat    %8s  %-13s%9.1f %-3s %-15s  %5.1f\n\n",carbprotfat,Nutrient[FIBTG],food_work.nutrient[FIBTG],Unit[FIBTG],"Unsat/Sat Ratio",food_work.nutrient[FASAT] == 0 ? 0 : (food_work.nutrient[FAMS] + food_work.nutrient[FAPU]) / food_work.nutrient[FASAT]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FAT],food_work.nutrient[FAT],Unit[FAT],Nutrient[VITA_IU],food_work.nutrient[VITA_IU],Unit[VITA_IU],Nutrient[CA],food_work.nutrient[CA],Unit[CA]);
  fprintf(fp," %-12s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FASAT],food_work.nutrient[FASAT],Unit[FASAT],Nutrient[THIA],food_work.nutrient[THIA],Unit[THIA],Nutrient[CU],food_work.nutrient[CU],Unit[CU]);
  fprintf(fp," %-12s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FAMS],food_work.nutrient[FAMS],Unit[FAMS],Nutrient[RIBF],food_work.nutrient[RIBF],Unit[RIBF],Nutrient[FE],food_work.nutrient[FE],Unit[FE]);
  fprintf(fp," %-12s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FAPU],food_work.nutrient[FAPU],Unit[FAPU],Nutrient[NIA],food_work.nutrient[NIA],Unit[NIA],Nutrient[MG],food_work.nutrient[MG],Unit[MG]);
  fprintf(fp,"  %-11s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[OMEGA6],food_work.nutrient[OMEGA6],Unit[OMEGA6],Nutrient[PANTAC],food_work.nutrient[PANTAC],Unit[PANTAC],Nutrient[MN],food_work.nutrient[MN],Unit[MN]);
  fprintf(fp,"    %-9s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[LA],food_work.nutrient[LA],Unit[LA],Nutrient[VITB6A],food_work.nutrient[VITB6A],Unit[VITB6A],Nutrient[P],food_work.nutrient[P],Unit[P]);
  fprintf(fp,"    %-9s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[AA],food_work.nutrient[AA],Unit[AA],Nutrient[FOL],food_work.nutrient[FOL],Unit[FOL],Nutrient[K],food_work.nutrient[K],Unit[K]);
  fprintf(fp,"  %-11s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[OMEGA3],food_work.nutrient[OMEGA3],Unit[OMEGA3],Nutrient[VITB12],food_work.nutrient[VITB12],Unit[VITB12],Nutrient[SE],food_work.nutrient[SE],Unit[SE]);
  fprintf(fp,"    %-9s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[ALA],food_work.nutrient[ALA],Unit[ALA],Nutrient[VITC],food_work.nutrient[VITC],Unit[VITC],Nutrient[NA],food_work.nutrient[NA],Unit[NA]);
  fprintf(fp,"    %-9s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[EPA],food_work.nutrient[EPA],Unit[EPA],Nutrient[VITD],food_work.nutrient[VITD],Unit[VITD],Nutrient[ZN],food_work.nutrient[ZN],Unit[ZN]);
  fprintf(fp,"    %-9s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[DHA],food_work.nutrient[DHA],Unit[DHA],Nutrient[VITE],food_work.nutrient[VITE],Unit[VITE]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[CHOLE],food_work.nutrient[CHOLE],Unit[CHOLE],Nutrient[VITK],food_work.nutrient[VITK],Unit[VITK]);
#else
  fprintf(fp,"%-13s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[ENERC_KCAL],food_work.nutrient[ENERC_KCAL],Unit[ENERC_KCAL],Nutrient[CHOCDF],food_work.nutrient[CHOCDF],Unit[CHOCDF],Nutrient[PROCNT],food_work.nutrient[PROCNT],Unit[PROCNT]);
  fprintf(fp,"Carb/Prot/Fat    %8s %-13s%9.1f %-3s %-15s  %5.1f\n\n",carbprotfat,Nutrient[FIBTG],food_work.nutrient[FIBTG],Unit[FIBTG],"Unsat/Sat Ratio",food_work.nutrient[FASAT] == 0 ? 0 : (food_work.nutrient[FAMS] + food_work.nutrient[FAPU]) / food_work.nutrient[FASAT]);
  fprintf(fp,"%-13s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FAT],food_work.nutrient[FAT],Unit[FAT],Nutrient[VITA_IU],food_work.nutrient[VITA_IU],Unit[VITA_IU],Nutrient[CA],food_work.nutrient[CA],Unit[CA]);
  fprintf(fp," %-12s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FASAT],food_work.nutrient[FASAT],Unit[FASAT],Nutrient[THIA],food_work.nutrient[THIA],Unit[THIA],Nutrient[CU],food_work.nutrient[CU],Unit[CU]);
  fprintf(fp," %-12s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FAMS],food_work.nutrient[FAMS],Unit[FAMS],Nutrient[RIBF],food_work.nutrient[RIBF],Unit[RIBF],Nutrient[FE],food_work.nutrient[FE],Unit[FE]);
  fprintf(fp," %-12s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FAPU],food_work.nutrient[FAPU],Unit[FAPU],Nutrient[NIA],food_work.nutrient[NIA],Unit[NIA],Nutrient[MG],food_work.nutrient[MG],Unit[MG]);
  fprintf(fp,"  %-11s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[OMEGA6],food_work.nutrient[OMEGA6],Unit[OMEGA6],Nutrient[PANTAC],food_work.nutrient[PANTAC],Unit[PANTAC],Nutrient[MN],food_work.nutrient[MN],Unit[MN]);
  fprintf(fp,"    %-9s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[LA],food_work.nutrient[LA],Unit[LA],Nutrient[VITB6A],food_work.nutrient[VITB6A],Unit[VITB6A],Nutrient[P],food_work.nutrient[P],Unit[P]);
  fprintf(fp,"    %-9s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[AA],food_work.nutrient[AA],Unit[AA],Nutrient[FOL],food_work.nutrient[FOL],Unit[FOL],Nutrient[K],food_work.nutrient[K],Unit[K]);
  fprintf(fp,"  %-11s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[OMEGA3],food_work.nutrient[OMEGA3],Unit[OMEGA3],Nutrient[VITB12],food_work.nutrient[VITB12],Unit[VITB12],Nutrient[SE],food_work.nutrient[SE],Unit[SE]);
  fprintf(fp,"    %-9s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[ALA],food_work.nutrient[ALA],Unit[ALA],Nutrient[VITC],food_work.nutrient[VITC],Unit[VITC],Nutrient[NA],food_work.nutrient[NA],Unit[NA]);
  fprintf(fp,"    %-9s%9.1f %-3s%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[EPA],food_work.nutrient[EPA],Unit[EPA],Nutrient[VITD],food_work.nutrient[VITD],Unit[VITD],Nutrient[ZN],food_work.nutrient[ZN],Unit[ZN]);
  fprintf(fp,"    %-9s%9.1f %-3s%-13s%9.1f %-3s\n",Nutrient[DHA],food_work.nutrient[DHA],Unit[DHA],Nutrient[VITE],food_work.nutrient[VITE],Unit[VITE]);
  fprintf(fp,"%-13s%9.1f %-3s%-13s%9.1f %-3s\n",Nutrient[CHOLE],food_work.nutrient[CHOLE],Unit[CHOLE],Nutrient[VITK],food_work.nutrient[VITK],Unit[VITK]);
#endif
  break;
 case 2 : 
#ifndef DOS
  fprintf(fp,"Carbohydrates              Amino Acids \n\n");
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[CHOCDF],food_work.nutrient[CHOCDF],Unit[CHOCDF],Nutrient[PROCNT],food_work.nutrient[PROCNT],Unit[PROCNT],Nutrient[LEU_G],food_work.nutrient[LEU_G],Unit[LEU_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FIBTG],food_work.nutrient[FIBTG],Unit[FIBTG],Nutrient[ADPROT],food_work.nutrient[ADPROT],Unit[ADPROT],Nutrient[LYS_G],food_work.nutrient[LYS_G],Unit[LYS_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[STARCH],food_work.nutrient[STARCH],Unit[STARCH],Nutrient[ALA_G],food_work.nutrient[ALA_G],Unit[ALA_G],Nutrient[MET_G],food_work.nutrient[MET_G],Unit[MET_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[SUGAR],food_work.nutrient[SUGAR],Unit[SUGAR],Nutrient[ARG_G],food_work.nutrient[ARG_G],Unit[ARG_G],Nutrient[PHE_G],food_work.nutrient[PHE_G],Unit[PHE_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FRUS],food_work.nutrient[FRUS],Unit[FRUS],Nutrient[ASP_G],food_work.nutrient[ASP_G],Unit[ASP_G],Nutrient[PRO_G],food_work.nutrient[PRO_G],Unit[PRO_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[GALS],food_work.nutrient[GALS],Unit[GALS],Nutrient[CYS_G],food_work.nutrient[CYS_G],Unit[CYS_G],Nutrient[SER_G],food_work.nutrient[SER_G],Unit[SER_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[GLUS],food_work.nutrient[GLUS],Unit[GLUS],Nutrient[GLU_G],food_work.nutrient[GLU_G],Unit[GLU_G],Nutrient[THR_G],food_work.nutrient[THR_G],Unit[THR_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[LACS],food_work.nutrient[LACS],Unit[LACS],Nutrient[GLY_G],food_work.nutrient[GLY_G],Unit[GLY_G],Nutrient[TRP_G],food_work.nutrient[TRP_G],Unit[TRP_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s %-13s%9.1f %-3s\n",Nutrient[MALS],food_work.nutrient[MALS],Unit[MALS],Nutrient[HISTN_G],food_work.nutrient[HISTN_G],Unit[HISTN_G],Nutrient[TYR_G],food_work.nutrient[TYR_G],Unit[TYR_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s %-13s%9.1f %-3s\n",Nutrient[SUCS],food_work.nutrient[SUCS],Unit[SUCS],Nutrient[HYP],food_work.nutrient[HYP],Unit[HYP],Nutrient[VAL_G],food_work.nutrient[VAL_G],Unit[VAL_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n\n",Nutrient[ALC],food_work.nutrient[ALC],Unit[ALC],Nutrient[ILE_G],food_work.nutrient[ILE_G],Unit[ILE_G]);
  fprintf(fp,"Note:  \"0.0\" may mean no data.\n");
#else
  fprintf(fp,"Carbohydrates              Amino Acids \n\n");
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[CHOCDF],food_work.nutrient[CHOCDF],Unit[CHOCDF],Nutrient[PROCNT],food_work.nutrient[PROCNT],Unit[PROCNT],Nutrient[LEU_G],food_work.nutrient[LEU_G],Unit[LEU_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[FIBTG],food_work.nutrient[FIBTG],Unit[FIBTG],Nutrient[ADPROT],food_work.nutrient[ADPROT],Unit[ADPROT],Nutrient[LYS_G],food_work.nutrient[LYS_G],Unit[LYS_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[STARCH],food_work.nutrient[STARCH],Unit[STARCH],Nutrient[ALA_G],food_work.nutrient[ALA_G],Unit[ALA_G],Nutrient[MET_G],food_work.nutrient[MET_G],Unit[MET_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[SUGAR],food_work.nutrient[SUGAR],Unit[SUGAR],Nutrient[ARG_G],food_work.nutrient[ARG_G],Unit[ARG_G],Nutrient[PHE_G],food_work.nutrient[PHE_G],Unit[PHE_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[FRUS],food_work.nutrient[FRUS],Unit[FRUS],Nutrient[ASP_G],food_work.nutrient[ASP_G],Unit[ASP_G],Nutrient[PRO_G],food_work.nutrient[PRO_G],Unit[PRO_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[GALS],food_work.nutrient[GALS],Unit[GALS],Nutrient[CYS_G],food_work.nutrient[CYS_G],Unit[CYS_G],Nutrient[SER_G],food_work.nutrient[SER_G],Unit[SER_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[GLUS],food_work.nutrient[GLUS],Unit[GLUS],Nutrient[GLU_G],food_work.nutrient[GLU_G],Unit[GLU_G],Nutrient[THR_G],food_work.nutrient[THR_G],Unit[THR_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[LACS],food_work.nutrient[LACS],Unit[LACS],Nutrient[GLY_G],food_work.nutrient[GLY_G],Unit[GLY_G],Nutrient[TRP_G],food_work.nutrient[TRP_G],Unit[TRP_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s %-13s%8.1f %-3s\n",Nutrient[MALS],food_work.nutrient[MALS],Unit[MALS],Nutrient[HISTN_G],food_work.nutrient[HISTN_G],Unit[HISTN_G],Nutrient[TYR_G],food_work.nutrient[TYR_G],Unit[TYR_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s %-13s%8.1f %-3s\n",Nutrient[SUCS],food_work.nutrient[SUCS],Unit[SUCS],Nutrient[HYP],food_work.nutrient[HYP],Unit[HYP],Nutrient[VAL_G],food_work.nutrient[VAL_G],Unit[VAL_G]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n\n",Nutrient[ALC],food_work.nutrient[ALC],Unit[ALC],Nutrient[ILE_G],food_work.nutrient[ILE_G],Unit[ILE_G]);
  fprintf(fp,"Note:  \"0.0\" may mean no data.\n");
#endif
  break;
 case 3 :
#ifndef DOS
  fprintf(fp,"Miscellaneous Nutrients\n\n");
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[ENERC_KJ],food_work.nutrient[ENERC_KJ],Unit[ENERC_KJ],Nutrient[TOCPHA],food_work.nutrient[TOCPHA],Unit[TOCPHA],Nutrient[CHOLE],food_work.nutrient[CHOLE],Unit[CHOLE]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[ASH],food_work.nutrient[ASH],Unit[ASH],Nutrient[TOCPHB],food_work.nutrient[TOCPHB],Unit[TOCPHB],Nutrient[PHYSTR],food_work.nutrient[PHYSTR],Unit[PHYSTR]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-14s%8.1f %-3s\n",Nutrient[WATER],food_work.nutrient[WATER],Unit[WATER],Nutrient[TOCPHG],food_work.nutrient[TOCPHG],Unit[TOCPHG],Nutrient[SITSTR],food_work.nutrient[SITSTR],Unit[SITSTR]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[CAFFN],food_work.nutrient[CAFFN],Unit[CAFFN],Nutrient[TOCPHD],food_work.nutrient[TOCPHD],Unit[TOCPHD],Nutrient[CAMD5],food_work.nutrient[CAMD5],Unit[CAMD5]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[THEBRN],food_work.nutrient[THEBRN],Unit[THEBRN],Nutrient[CARTA],food_work.nutrient[CARTA],Unit[CARTA],Nutrient[STID7],food_work.nutrient[STID7],Unit[STID7]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FOLAC],food_work.nutrient[FOLAC],Unit[FOLAC],Nutrient[CARTB],food_work.nutrient[CARTB],Unit[CARTB]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s\n",Nutrient[FOLFD],food_work.nutrient[FOLFD],Unit[FOLFD],Nutrient[CRYPX],food_work.nutrient[CRYPX],Unit[CRYPX]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s\n",Nutrient[FOLDFE],food_work.nutrient[FOLDFE],Unit[FOLDFE],Nutrient[LUT_ZEA],food_work.nutrient[LUT_ZEA],Unit[LUT_ZEA]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[RETOL],food_work.nutrient[RETOL],Unit[RETOL],Nutrient[LYCPN],food_work.nutrient[LYCPN],Unit[LYCPN]);
  fprintf(fp,"%-13s%9.1f %-3s\n\n\n",Nutrient[VITA_RAE],food_work.nutrient[VITA_RAE],Unit[VITA_RAE]);
  fprintf(fp,"Note:  \"0.0\" may mean no data.\n");
#else
  fprintf(fp,"Miscellaneous Nutrients\n\n");
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[ENERC_KJ],food_work.nutrient[ENERC_KJ],Unit[ENERC_KJ],Nutrient[TOCPHA],food_work.nutrient[TOCPHA],Unit[TOCPHA],Nutrient[CHOLE],food_work.nutrient[CHOLE],Unit[CHOLE]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[ASH],food_work.nutrient[ASH],Unit[ASH],Nutrient[TOCPHB],food_work.nutrient[TOCPHB],Unit[TOCPHB],Nutrient[PHYSTR],food_work.nutrient[PHYSTR],Unit[PHYSTR]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-14s%7.1f %-3s\n",Nutrient[WATER],food_work.nutrient[WATER],Unit[WATER],Nutrient[TOCPHG],food_work.nutrient[TOCPHG],Unit[TOCPHG],Nutrient[SITSTR],food_work.nutrient[SITSTR],Unit[SITSTR]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[CAFFN],food_work.nutrient[CAFFN],Unit[CAFFN],Nutrient[TOCPHD],food_work.nutrient[TOCPHD],Unit[TOCPHD],Nutrient[CAMD5],food_work.nutrient[CAMD5],Unit[CAMD5]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s %-13s%8.1f %-3s\n",Nutrient[THEBRN],food_work.nutrient[THEBRN],Unit[THEBRN],Nutrient[CARTA],food_work.nutrient[CARTA],Unit[CARTA],Nutrient[STID7],food_work.nutrient[STID7],Unit[STID7]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FOLAC],food_work.nutrient[FOLAC],Unit[FOLAC],Nutrient[CARTB],food_work.nutrient[CARTB],Unit[CARTB]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s\n",Nutrient[FOLFD],food_work.nutrient[FOLFD],Unit[FOLFD],Nutrient[CRYPX],food_work.nutrient[CRYPX],Unit[CRYPX]);
  fprintf(fp,"%-13s%9.1f %-3s %-14s%8.1f %-3s\n",Nutrient[FOLDFE],food_work.nutrient[FOLDFE],Unit[FOLDFE],Nutrient[LUT_ZEA],food_work.nutrient[LUT_ZEA],Unit[LUT_ZEA]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[RETOL],food_work.nutrient[RETOL],Unit[RETOL],Nutrient[LYCPN],food_work.nutrient[LYCPN],Unit[LYCPN]);
  fprintf(fp,"%-13s%9.1f %-3s\n\n\n",Nutrient[VITA_RAE],food_work.nutrient[VITA_RAE],Unit[VITA_RAE]);
  fprintf(fp,"Note:  \"0.0\" may mean no data.\n");
#endif
  break;
 case 4 :
  fprintf(fp,"Saturated Fatty Acids\n\n");
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[FASAT],food_work.nutrient[FASAT],Unit[FASAT],Nutrient[F15D0],food_work.nutrient[F15D0],Unit[F15D0]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[F4D0],food_work.nutrient[F4D0],Unit[F4D0],Nutrient[F16D0],food_work.nutrient[F16D0],Unit[F16D0]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[F6D0],food_work.nutrient[F6D0],Unit[F6D0],Nutrient[F17D0],food_work.nutrient[F17D0],Unit[F17D0]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[F8D0],food_work.nutrient[F8D0],Unit[F8D0],Nutrient[F18D0],food_work.nutrient[F18D0],Unit[F18D0]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[F10D0],food_work.nutrient[F10D0],Unit[F10D0],Nutrient[F20D0],food_work.nutrient[F20D0],Unit[F20D0]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[F12D0],food_work.nutrient[F12D0],Unit[F12D0],Nutrient[F22D0],food_work.nutrient[F22D0],Unit[F22D0]);
  fprintf(fp,"%-13s%9.1f %-3s %-13s%9.1f %-3s\n",Nutrient[F13D0],food_work.nutrient[F13D0],Unit[F13D0],Nutrient[F24D0],food_work.nutrient[F24D0],Unit[F24D0]);
  fprintf(fp,"%-13s%9.1f %-3s\n\n\n\n\n",Nutrient[F14D0],food_work.nutrient[F14D0],Unit[F14D0]);
  fprintf(fp,"Note:  \"0.0\" may mean no data.\n");
  break;
 case 5 :
#ifndef DOS
  fprintf(fp,"Unsaturated Fatty Acids    %-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[FAPU],food_work.nutrient[FAPU],Unit[FAPU],Nutrient[FATRN],food_work.nutrient[FATRN],Unit[FATRN]);
  fprintf(fp,"                           %-13s%9.2f %-3s %-14s%8.2f %-3s\n",Nutrient[F18D2],food_work.nutrient[F18D2],Unit[F18D2],Nutrient[FATRNM],food_work.nutrient[FATRNM],Unit[FATRNM]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[FAMS],food_work.nutrient[FAMS],Unit[FAMS],Nutrient[F18D2CN6],food_work.nutrient[F18D2CN6],Unit[F18D2CN6],Nutrient[F16D1T],food_work.nutrient[F16D1T],Unit[F16D1T]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F14D1],food_work.nutrient[F14D1],Unit[F14D1],Nutrient[F18D3],food_work.nutrient[F18D3],Unit[F18D3],Nutrient[F18D1T],food_work.nutrient[F18D1T],Unit[F18D1T]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-14s%8.2f %-3s\n",Nutrient[F15D1],food_work.nutrient[F15D1],Unit[F15D1],Nutrient[F18D3CN3],food_work.nutrient[F18D3CN3],Unit[F18D3CN3],Nutrient[FATRNP],food_work.nutrient[FATRNP],Unit[FATRNP]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F16D1],food_work.nutrient[F16D1],Unit[F16D1],Nutrient[F18D3CN6],food_work.nutrient[F18D3CN6],Unit[F18D3CN6],Nutrient[F18D2I],food_work.nutrient[F18D2I],Unit[F18D2I]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F16D1C],food_work.nutrient[F16D1C],Unit[F16D1C],Nutrient[F18D4],food_work.nutrient[F18D4],Unit[F18D4],Nutrient[F18D2CLA],food_work.nutrient[F18D2CLA],Unit[F18D2CLA]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F17D1],food_work.nutrient[F17D1],Unit[F17D1],Nutrient[F20D2CN6],food_work.nutrient[F20D2CN6],Unit[F20D2CN6]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F18D1],food_work.nutrient[F18D1],Unit[F18D1],Nutrient[F20D3],food_work.nutrient[F20D3],Unit[F20D3]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F18D1C],food_work.nutrient[F18D1C],Unit[F18D1C],Nutrient[F20D4],food_work.nutrient[F20D4],Unit[F20D4]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F20D1],food_work.nutrient[F20D1],Unit[F20D1],Nutrient[F20D5],food_work.nutrient[F20D5],Unit[F20D5]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F22D1],food_work.nutrient[F22D1],Unit[F22D1],Nutrient[F22D5],food_work.nutrient[F22D5],Unit[F22D5]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F24D1C],food_work.nutrient[F24D1C],Unit[F24D1C],Nutrient[F22D6],food_work.nutrient[F22D6],Unit[F22D6]);
  fprintf(fp,"\nNote:  \"0.00\" may mean no data.\n");
#else
  fprintf(fp,"Unsaturated Fatty Acids    %-13s%9.2f %-3s %-13s%8.2f %-3s\n",Nutrient[FAPU],food_work.nutrient[FAPU],Unit[FAPU],Nutrient[FATRN],food_work.nutrient[FATRN],Unit[FATRN]);
  fprintf(fp,"                           %-13s%9.2f %-3s %-14s%7.2f %-3s\n",Nutrient[F18D2],food_work.nutrient[F18D2],Unit[F18D2],Nutrient[FATRNM],food_work.nutrient[FATRNM],Unit[FATRNM]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%8.2f %-3s\n",Nutrient[FAMS],food_work.nutrient[FAMS],Unit[FAMS],Nutrient[F18D2CN6],food_work.nutrient[F18D2CN6],Unit[F18D2CN6],Nutrient[F16D1T],food_work.nutrient[F16D1T],Unit[F16D1T]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%8.2f %-3s\n",Nutrient[F14D1],food_work.nutrient[F14D1],Unit[F14D1],Nutrient[F18D3],food_work.nutrient[F18D3],Unit[F18D3],Nutrient[F18D1T],food_work.nutrient[F18D1T],Unit[F18D1T]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-14s%7.2f %-3s\n",Nutrient[F15D1],food_work.nutrient[F15D1],Unit[F15D1],Nutrient[F18D3CN3],food_work.nutrient[F18D3CN3],Unit[F18D3CN3],Nutrient[FATRNP],food_work.nutrient[FATRNP],Unit[FATRNP]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%8.2f %-3s\n",Nutrient[F16D1],food_work.nutrient[F16D1],Unit[F16D1],Nutrient[F18D3CN6],food_work.nutrient[F18D3CN6],Unit[F18D3CN6],Nutrient[F18D2I],food_work.nutrient[F18D2I],Unit[F18D2I]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s %-13s%8.2f %-3s\n",Nutrient[F16D1C],food_work.nutrient[F16D1C],Unit[F16D1C],Nutrient[F18D4],food_work.nutrient[F18D4],Unit[F18D4],Nutrient[F18D2CLA],food_work.nutrient[F18D2CLA],Unit[F18D2CLA]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F17D1],food_work.nutrient[F17D1],Unit[F17D1],Nutrient[F20D2CN6],food_work.nutrient[F20D2CN6],Unit[F20D2CN6]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F18D1],food_work.nutrient[F18D1],Unit[F18D1],Nutrient[F20D3],food_work.nutrient[F20D3],Unit[F20D3]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F18D1C],food_work.nutrient[F18D1C],Unit[F18D1C],Nutrient[F20D4],food_work.nutrient[F20D4],Unit[F20D4]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F20D1],food_work.nutrient[F20D1],Unit[F20D1],Nutrient[F20D5],food_work.nutrient[F20D5],Unit[F20D5]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F22D1],food_work.nutrient[F22D1],Unit[F22D1],Nutrient[F22D5],food_work.nutrient[F22D5],Unit[F22D5]);
  fprintf(fp,"%-13s%9.2f %-3s %-13s%9.2f %-3s\n",Nutrient[F24D1C],food_work.nutrient[F24D1C],Unit[F24D1C],Nutrient[F22D6],food_work.nutrient[F22D6],Unit[F22D6]);
  fprintf(fp,"\nNote:  \"0.00\" may mean no data.\n");
#endif
  break;
 default :
  fprintf(fp,"Calories(%4.0f)   %5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",DV[ENERC_KCAL],food_work.nutrient[ENERC_KCAL]/DV[ENERC_KCAL]*100,Nutrient[CHOCDF],food_work.nutrient[CHOCDF]/DV[CHOCDF]*100,Nutrient[PROCNT],food_work.nutrient[PROCNT]/DV[PROCNT]*100);
  fprintf(fp,"Carb/Prot/Fat  %8s     %-13s    %5.0f%%     %-15s   %5.1f\n\n",carbprotfat,Nutrient[FIBTG],food_work.nutrient[FIBTG]/DV[FIBTG]*100,"Unsat/Sat Ratio",food_work.nutrient[FASAT] == 0 ? 0 : (food_work.nutrient[FAMS] + food_work.nutrient[FAPU]) / food_work.nutrient[FASAT]);
  fprintf(fp,"%-13s    %5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[FAT],food_work.nutrient[FAT]/DV[FAT]*100,Nutrient[VITA_IU],food_work.nutrient[VITA_IU]/DV[VITA_IU]*100,Nutrient[CA],food_work.nutrient[CA]/DV[CA]*100);
  fprintf(fp," %-13s   %5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[FASAT],food_work.nutrient[FASAT]/DV[FASAT]*100,Nutrient[THIA],food_work.nutrient[THIA]/DV[THIA]*100,Nutrient[CU],food_work.nutrient[CU]/DV[CU]*100);
  fprintf(fp," %-13s %7.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[FAMS],food_work.nutrient[FAMS]/DV[FAMS]*100,Nutrient[RIBF],food_work.nutrient[RIBF]/DV[RIBF]*100,Nutrient[FE],food_work.nutrient[FE]/DV[FE]*100);
  fprintf(fp," %-13s   %5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[FAPU],food_work.nutrient[FAPU]/DV[FAPU]*100,Nutrient[NIA],food_work.nutrient[NIA]/DV[NIA]*100,Nutrient[MG],food_work.nutrient[MG]/DV[MG]*100);
  fprintf(fp,"  %-13s  %5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[OMEGA6],food_work.nutrient[OMEGA6]/DV[OMEGA6]*100,Nutrient[PANTAC],food_work.nutrient[PANTAC]/DV[PANTAC]*100,Nutrient[MN],food_work.nutrient[MN]/DV[MN]*100);
  fprintf(fp,"    %-13s%5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[LA],food_work.nutrient[LA]/DV[LA]*100,Nutrient[VITB6A],food_work.nutrient[VITB6A]/DV[VITB6A]*100,Nutrient[P],food_work.nutrient[P]/DV[P]*100);
  fprintf(fp,"    %-13s%5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[AA],food_work.nutrient[AA]/DV[AA]*100,Nutrient[FOL],food_work.nutrient[FOL]/DV[FOL]*100,Nutrient[K],food_work.nutrient[K]/DV[K]*100);
  fprintf(fp,"  %-13s  %5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[OMEGA3],food_work.nutrient[OMEGA3]/DV[OMEGA3]*100,Nutrient[VITB12],food_work.nutrient[VITB12]/DV[VITB12]*100,Nutrient[SE],food_work.nutrient[SE]/DV[SE]*100);
  fprintf(fp,"    %-13s%5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[ALA],food_work.nutrient[ALA]/DV[ALA]*100,Nutrient[VITC],food_work.nutrient[VITC]/DV[VITC]*100,Nutrient[NA],food_work.nutrient[NA]/DV[NA]*100);
  fprintf(fp,"    %-13s%5.0f%%     %-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[EPA],food_work.nutrient[EPA]/DV[EPA]*100,Nutrient[VITD],food_work.nutrient[VITD]/DV[VITD]*100,Nutrient[ZN],food_work.nutrient[ZN]/DV[ZN]*100);
  fprintf(fp,"    %-13s%5.0f%%     %-13s    %5.0f%%\n",Nutrient[DHA],food_work.nutrient[DHA]/DV[DHA]*100,Nutrient[VITE],food_work.nutrient[VITE]/DV[VITE]*100);
  fprintf(fp,"%-13s    %5.0f%%     %-13s    %5.0f%%\n",Nutrient[CHOLE],food_work.nutrient[CHOLE]/DV[CHOLE]*100,Nutrient[VITK],food_work.nutrient[VITK]/DV[VITK]*100);
  break;
 }
}

void order_new_recipe()
{
struct food *food_ptr = &recipe_root;
while (food_ptr->next != NULL) food_ptr = food_ptr->next;
new_recipe->next = NULL;
food_ptr->next = new_recipe;
}

void delete_recipe_food(int num)
{
struct food *m, *recipe_ptr = &recipe_root;
int count = 0;
while (recipe_ptr->next != NULL)
 {
 count++;
 if (count == num)
  {
  m = recipe_ptr->next;
  recipe_ptr->next = recipe_ptr->next->next;
  free(m);
  return;
  }
 recipe_ptr = recipe_ptr->next;
 }
}

void compute_derived_fields(struct food *food_ptr)
{
float ratio;
food_ptr->nutrient[VITE] = 1.5 * food_ptr->nutrient[TOCPHA];
food_ptr->nutrient[LA] = food_ptr->nutrient[F18D2CN6] > 0 ? food_ptr->nutrient[F18D2CN6] : food_ptr->nutrient[F18D2] - food_ptr->nutrient[F18D2I] - food_ptr->nutrient[F18D2CLA];
food_ptr->nutrient[OMEGA6] = food_ptr->nutrient[LA];
food_ptr->nutrient[ALA] = food_ptr->nutrient[F18D3CN3] > 0 ? food_ptr->nutrient[F18D3CN3] : food_ptr->nutrient[F18D3] - food_ptr->nutrient[F18D3CN6];
food_ptr->nutrient[OMEGA3] = food_ptr->nutrient[ALA];
food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F18D4];
food_ptr->nutrient[OMEGA6] += food_ptr->nutrient[F20D2CN6];
food_ptr->nutrient[OMEGA6] += food_ptr->nutrient[F20D3];
food_ptr->nutrient[OMEGA6] += food_ptr->nutrient[F20D4];
food_ptr->nutrient[AA] = food_ptr->nutrient[F20D4];
food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F20D5];
food_ptr->nutrient[EPA] = food_ptr->nutrient[F20D5];
food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F22D5];
food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F22D6];
food_ptr->nutrient[DHA] = food_ptr->nutrient[F22D6];

if ( food_ptr->prot_cal_factor == 0 && food_ptr->fat_cal_factor == 0 && food_ptr->cho_cal_factor == 0 && food_ptr->nutrient[ENERC_KCAL] > 0)
 {
 ratio = ((food_ptr->nutrient[PROCNT] * 4) + (food_ptr->nutrient[FAT] * 9) + (food_ptr->nutrient[CHOCDF] * 4)) / food_ptr->nutrient[ENERC_KCAL];
 if (ratio != 0)
  {
  food_ptr->nutrient[PROT_KCAL] = 4 * food_ptr->nutrient[PROCNT] / ratio;
  food_ptr->nutrient[FAT_KCAL] = 9 * food_ptr->nutrient[FAT] / ratio;
  food_ptr->nutrient[CHO_KCAL] = 4 * food_ptr->nutrient[CHOCDF] / ratio;
  }
 } 
else
 {
 food_ptr->nutrient[PROT_KCAL] = food_ptr->prot_cal_factor * food_ptr->nutrient[PROCNT];
 food_ptr->nutrient[FAT_KCAL] = food_ptr->fat_cal_factor * food_ptr->nutrient[FAT];
 food_ptr->nutrient[CHO_KCAL] = food_ptr->cho_cal_factor * food_ptr->nutrient[CHOCDF];
 }
}
